#!/usr/bin/env python3
"""
CommandLine interface for test.

For checking tests call::

    python -m %MODULE%.tests
"""

# pylint: disable=W0611
from __future__ import unicode_literals
import unittest
from %MODULE%.tests import Test%MODULE%Module


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
""" Module to manage template's arguments. """


class ArgumentError(Exception):
    """ Argument error. """


class Argument:
    """ One of argument. """
    def __init__(self,
                 name,
                 required=True,
                 default="",
                 question=None,
                 description=None):
        self.__name = name.lower()
        self.__required = required
        self.default = default if default is not None else ""
        default_question = "Put the '{value}' value"
        self.__question = question if question is not None and question != "" else default_question
        self.description = description if description is not None and description != "" \
            else "Value of '{}'".format(name)
        if self.default != "":
            self.description = "{} (default: {})".format(self.description, self.default)
        self.__value = None

    @staticmethod
    def create_from_dict(dictionary):
        """ Create argument from dictionary element. """
        assert isinstance(dictionary, dict)
        if 'name' not in dictionary:
            raise ArgumentError("Invalid format of argument object")
        name = dictionary['name']
        required = dictionary['required'] if 'required' in dictionary else True
        default = dictionary['default'] if 'default' in dictionary else ""
        question = dictionary['question'] if 'question' in dictionary else None
        description = dictionary['description'] if 'description' in dictionary else None
        return Argument(name, required, default, question, description)

    @property
    def name(self):
        """ Name of argument. """
        return self.__name.upper()

    @property
    def value(self):
        """ Value of argument. """
        if self.__value is None:
            return self.default
        return self.__value

    @value.setter
    def value(self, value):
        """ Set value. """
        self.__value = value

    @property
    def is_set(self):
        """ True if set, False otherwise. """
        if not self.__required:
            return True
        return self.__value is not None

    @property
    def question(self):
        """ String of question about value. """
        return self.__question.format(value=self.name)

    def __str__(self):
        return self.__name


class ArgumentsContainer:
    """ Manager with information about arguments. """
    def __init__(self, args):
        self.__arguments = {}
        if args is None:
            return
        if isinstance(args, list):
            for key in args:
                self.__add_argument(key)
        else:
            raise ArgumentError("Cannot add element type: {}".format(type(args)))

    def __add_argument(self, arg, value=None):
        """ Add one argument. """
        new_element = Argument.create_from_dict(arg)
        if new_element.name in self.__arguments:
            raise ArgumentError("Cannot add element {} twice".format(new_element.name))
        if value is not None:
            new_element.value = value
        self.__arguments[new_element.name.lower()] = new_element

    @property
    def missing_args(self):
        """ Return all argument that is not set. """
        result = []
        for key in self.__arguments:
            if not self.__arguments[key].is_set:
                result.append(self.__arguments[key])
        return result

    @property
    def all(self):
        """ List of all elements. """
        return self.__arguments

    def input_missing(self):
        """ Ask about all missing arguments. """
        for argument in self.missing_args:
            input_str = input("{}: ".format(argument.question))
            argument.value = input_str

    def __update_value(self, key, value):
        """ Update value of existing atribute or add new one. """
        if key.lower() in self.__arguments:
            self.__arguments[key.lower()].value = value
        else:
            new_argument = Argument(key)
            new_argument.value = value
            self.__arguments[key.lower()] = new_argument

    def add_values(self, list_of_values):
        """ Add value of element. """
        assert isinstance(list_of_values, dict)
        for key in list_of_values:
            self.__update_value(key, list_of_values[key])

    def add_values_from_list(self, list_of_arguments):
        """ Add values from list of 2-elements lists. """
        assert isinstance(list_of_arguments, list)
        for argument_elem in list_of_arguments:
            assert isinstance(argument_elem, list) and len(argument_elem) == 2
            self.__update_value(argument_elem[0], argument_elem[1])

    def get_argument(self, key):
        """ Get argument by key. """
        return self.__arguments[key.lower()]

    def update(self, arguments):
        """ Update values of arguments. """
        assert isinstance(arguments, ArgumentsContainer) or arguments is None
        for key in arguments.all:
            if key in self.__arguments:
                this_arg = self.__arguments[key]
                if this_arg.default == "":
                    this_arg.default = arguments.all[key].default
            else:
                self.__arguments[key.lower()] = arguments.all[key]


def empty_args():
    """ Crate ArgumentContainer for empty arguments list. """
    return ArgumentsContainer(None)



#!/usr/bin/env python3
"""
Module to parsing manifest file.
"""

import os
import json
import logging
from templateme.arguments import ArgumentsContainer


class ManifestError(Exception):
    """ Manifest error. """


class Manifest:
    """ Manifest parsing class. """

    def _read_argument(self, key, default=""):
        if key in self.data:
            return self.data[key]
        return default

    def __init__(self, data, template):
        self.data = data

        self.short_description = self._read_argument("short-description", "No description")
        self.description = self._read_argument("description", "No description")
        self.include = self._read_argument("include", [])
        self.template = template
        if not isinstance(self.include, list):
            self.include = [self.include]

        self.args = ArgumentsContainer(self._read_argument("args", []))

    @staticmethod
    def create_from_file(path, template):
        """ Create manifest object from file. """
        if not os.path.isfile(path):
            raise ManifestError("Manifest not exist")
        try:
            with open(path) as data_file:
                data = json.load(data_file)
        except json.decoder.JSONDecodeError:
            logging.warning("Cannot read manifest file [%s]", data_file)
            raise ManifestError("Invalid manifest file")
        return Manifest(data, template)

    @staticmethod
    def create_from_string(text, template):
        """ Create manifest object from string. """
        data = json.loads(text)
        return Manifest(data, template)



#!/usr/bin/env python3
"""
Create new project in defined place..

The TemplateMe works as console application. It allows you to create new
project from predefined templates. To see list of all available elements
call:

::

    templateme --list

In case that you want to prepare new project for your favourite language
you need to choose one of the templates and write in your terminal:

::

    templateme -t <template_name> -o <your_project_name>

Where ``<template_name>`` is the one of the templates, listed in previous step
and ``<your_project_name>`` is a name of folder with your created project. After
that, the program will ask you to write a few variables that are necessary to
build templates.

"""

import logging
import sys
from argparse import ArgumentParser
from argparse import ArgumentTypeError
from templateme import get_version
from templateme.manager import TMPManager
from templateme.containers.abstract import Template
from templateme.containers.abstract import TemplateError


def __option_args(argv=None):
    """
    Parsing argument for command line program.

    @param argv: Option parameters
    @type argv: list
    @return: parsed arguments
    """
    def check_arguments(value):
        """ Check if argument value looks ok. """
        valuelist = value.split("=")
        if len(valuelist) != 2:
            raise ArgumentTypeError("Value should have format: argument=value")
        if valuelist[0].strip() == "" or valuelist[1].strip() == "":
            raise ArgumentTypeError("Value and argument name should not be empty")
        return valuelist

    parser = ArgumentParser(description="Create new project from template",
                            prog='templateme')
    parser.add_argument("-v", "--version", action='version',
                        version='%(prog)s {}'.format(get_version()))
    parser.add_argument("-d", "--log", dest="logLevel",
                        choices=['DEBUG', 'INFO', 'WARNING',
                                 'ERROR', 'CRITICAL'],
                        default='ERROR',
                        help="Set the logging level")
    parser.add_argument("-o", "--out", metavar="NAME",
                        dest="project_name", default=None,
                        help="Name of your new project directory")
    parser.add_argument("-l", "--list", action="store_true",
                        dest="list", default=False,
                        help="List available templates")
    parser.add_argument("-s", "--short-list", action="store_true",
                        dest="short_list", default=False,
                        help="Templates list wihtout description")
    parser.add_argument("-a", "--argument", action="append",
                        dest="argvalues", type=check_arguments,
                        default=[],
                        help="List of predefinied arguments")
    parser.add_argument("-q", "--quite", action="store_true",
                        dest="quite", default=False,
                        help="Not ask about arguments")
    parser.add_argument("-t", "--template", metavar="TEMPLATE",
                        dest="template", default="",
                        help="Template name")
    parser.add_argument("-f", "--force", action="store_true",
                        dest="force", default=False,
                        help="Force save template")
    return parser.parse_args(argv)


def print_template(template):
    """ Print information about template. """
    separator = "-----------------------"
    print("{sep}\nTemplate - {name}\n{sep}".format(sep=separator, name=template.name))
    print("{short_desc}\n\n{desc}\n{sep}"
          "".format(sep=separator,
                    short_desc=template.short_description,
                    desc=template.description))
    print("")
    for attribute in template.args.all.values():
        print(" - {} - {}".format(attribute.name, attribute.description))

    print("\n{}\n".format(separator))
    for elem in template.elements:
        print(" * ./{}".format(elem.path))
    print("\n{}".format(separator))


def examine_save(template, project_name, force):
    """
    Check if template exists and you can save it.
    If not, ask about confirmation to do this.
    """
    if force or project_name is None:
        return
    try:
        template.examine_save(project_name)
    except TemplateError:
        # Examine not pass, ask user of it
        input_str = input("File exists. Do you want override the template (y[es]|n[o]): ")
        if input_str in ("yes", "y", "ok"):
            print("Template will be overrided")
        elif input_str in ("no", "n"):
            print("We not override template")
            if input_str not in ("no", "n"):
                print("Next time use one of this answers: y[es]|n[o]")
            raise


def main(argv=None, debug=False):
    """
    Main function for command line program.

    @param argv: Option parameters
    @type argv: list
    """
    options = __option_args(argv)
    logging.basicConfig(format='%(asctime)s - %(name)s - '
                               '%(levelname)s - %(message)s',
                        level=options.logLevel)
    manager = TMPManager(options.project_name, debug=debug)

    if options.list or options.short_list:
        if options.template == "":
            templates = manager.get_all_templates()
            print("Here's a list of all templates:\n")
            for temp in templates:
                if options.short_list:
                    print("{} ".format(temp))
                else:
                    print(" * {} - {}".format(temp, temp.short_description))
            sys.exit(0)
        else:
            template = manager.get_template(options.template)
            print_template(template)
            sys.exit(0)
    elif options.template == "":
        print("You should define template's name\nSee --help for more information")
        sys.exit(1)

    template = manager.get_template(options.template)
    assert isinstance(template, Template)
    if template is None:
        print("There are not template name: ", options.template)
        sys.exit(1)
    force = options.force
    template.args.add_values_from_list(options.argvalues)
    try:
        if not options.quite:
            examine_save(template, options.project_name, force)

            # if examine was confirmed that you want to save template,
            # you can rewrite it even on existing files.
            force = True

            template.args.input_missing()
        template.save(options.project_name, options.project_name, force=force)
    except TemplateError as ex:
        print("Cannot save: ", ex)
        sys.exit(2)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
Testing module console.
"""

from __future__ import unicode_literals
import unittest
import sys
from io import StringIO
import mock
import templateme
from templateme.console import main as console_program


# This is tested class. Can have too many method
class TestConsoleModule(unittest.TestCase):  # pylint: disable=R0904
    """ Module testsCase. """

    def setUp(self):
        """ Setup environment. """
        self.old_stdout = sys.stdout
        self.old_stderr = sys.stderr
        sys.stdout = self.tmp_stdout = StringIO()
        sys.stderr = StringIO()

    def tearDown(self):
        """ Teardown environment. """
        sys.stdout = self.old_stdout
        sys.stderr = self.old_stderr

    def test_console(self):
        """ Test that console app work ok. """
        status = 0
        try:
            console_program(["--help"])
        except SystemExit as ex:
            if ex.code is not None:
                status = ex.code  # pylint disable=E0012,R0204
        self.assertEqual(status, 0)
        self.assertNotEqual(self.tmp_stdout.getvalue(), "")

    def test_invalid_args(self):
        """ Test invalid arguments. """
        status = 0
        try:
            console_program(["--invalid-arg", "--another-invalid-arg"])
        except SystemExit as ex:
            if ex.code is not None:
                status = ex.code  # pylint disable=E0012,R0204
        self.assertEqual(self.tmp_stdout.getvalue(), "")
        self.assertNotEqual(status, 0)

    def test_version(self):
        """ Test invalid arguments. """
        status = 0
        try:
            console_program(["--version"])
        except SystemExit as ex:
            if ex.code is not None:
                status = ex.code  # pylint disable=E0012,R0204
        self.assertEqual(status, 0)
        self.assertEqual(self.tmp_stdout.getvalue().strip(),
                         "templateme {}".format(templateme.get_version()))

    def test_listing(self):
        """ Testing list of default templates. """
        status = 0
        try:
            console_program(['--list'])
        except SystemExit as ex:
            if ex.code is not None:
                status = ex.code  # pylint disable=E0012, R0204
        self.assertEqual(status, 0)

    def test_create_template(self):
        """ Test if template works. """
        str_check = 'SomeStringTakePlaceInCode'
        status = 0
        try:
            with mock.patch('builtins.input', return_value=str_check):
                console_program(['-t', 'cpp'], debug=True)
        except SystemExit as ex:
            if ex.code is not None:
                status = ex.code  # pylint disable=E0012, R0204
        self.assertEqual(status, 0)
        self.assertTrue(self.tmp_stdout.getvalue().find(str_check) > -1)

    def test_create_with_arguments(self):
        """ Test if argument arg works. """
        str_check = 'SomeStringTakePlaceInCode'
        status = 0
        try:
            console_program(['-t', 'cpp', '-a', "class={}".format(str_check)], debug=True)
        except SystemExit as ex:
            if ex.code is not None:
                status = ex.code  # pylint disable=E0012, R0204
        self.assertEqual(status, 0)
        self.assertTrue(self.tmp_stdout.getvalue().find(str_check) > -1)

        for invalid_arg in ["value", "=", "a=", "=a", "a=a=v"]:
            try:
                console_program(['-t', 'cpp', '-a', "class=class",
                                 '-a', invalid_arg],
                                debug=True)
            except SystemExit as ex:
                if ex.code is not None:
                    status = ex.code  # pylint disable=E0012, R0204
            self.assertEqual(status, 2, "Invalid argument test result:({}=2), checked arg [{}]"
                             "".format(status, invalid_arg))

    def test_create_template_quiet(self):
        """ Test if template realy not ask in quiet mode. """
        status = 0
        try:
            console_program(['-t', 'cpp', '-q'], debug=True)
        except SystemExit as ex:
            if ex.code is not None:
                status = ex.code  # pylint disable=E0012, R0204
        self.assertEqual(status, 2)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
"""
Testing module templates.
"""

import unittest
from templateme.containers.abstract import TMPSource
from templateme.containers.abstract import Template


class TemplateTested(Template):
    """ Template object for testing. """

    def __init__(self, name, test_key):
        Template.__init__(self, name, None)
        self.test_key = test_key

    @classmethod
    def _get_elements(cls):
        return None


# This is tested class. Can have too many method
class TestTemplatesModule(unittest.TestCase):  # pylint: disable=R0904
    """ Module testsCase. """

    def test_source_template(self):
        """ Test if template source works ok. """
        class SourceTestTemplate(TMPSource):
            """ Tested source Template. """

            def __init__(self):
                TMPSource.__init__(self, None)

            def get_all_templates(self):
                return [TemplateTested("tested_temp", "test")]

        source = SourceTestTemplate()

        self.assertEqual(source.get_template("tested_temp").test_key, "test")
        self.assertIsNone(source.get_template("not_exist"))

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
"""
Module with abstract classes.
"""
import fnmatch
import abc
import os
import re
from templateme.arguments import empty_args


class TemplateError(Exception):
    """ Class describe template error. """


class TMPElement(abc.ABC):
    """ Class with one of files in template. """
    def __init__(self, path, template, project_name="project"):
        self._format = ""
        self.path = path
        self.template = template
        self.project_name = project_name

    @classmethod
    def load_txt(cls):
        """ Load text. """
        raise NotImplementedError

    @property
    def text(self):
        """ Element text. """
        if self._format == "":
            self._format = self.template.manager.render_template_txt(self.load_txt(), self.template)
        return self._format

    @property
    def save_path(self):
        """ Path to save element in output directory. """
        return self.template.manager.render_template_txt(self.path, self.template)

    def print_element(self):
        """ Print path and source of element. """
        print("{selector}\n{el_path}\n{selector}\n{el_source}\n{selector}\n\n"
              "".format(selector="--------",
                        el_path=self.save_path,
                        el_source=self.text))

    def save(self, path, project_name="project"):
        """ Save element in project. """
        save_path = os.path.join(path, self.save_path)
        save_path = re.sub(r"/^{}/".format(self.template.name), project_name, save_path)
        try:
            os.makedirs(os.path.dirname(save_path))
        except FileExistsError:
            pass
        file = open(save_path, "w")
        file.write(self.text)
        file.close()
        print("save file: ", save_path)


class Template(abc.ABC):
    """ Class which describe template. """
    def __init__(self, name, manager, manifest=None):
        self.manager = manager
        self._name = name

        self._ignored = ['manifest.json', '*.swp', "__pycache__", '*.pyc']
        self._include_templates = None
        self._all_args = None
        self._all_elements = None
        self.__args = None
        self.__args_updated = False
        self._includes = []
        self.manifest = manifest
        if self.manifest:
            for include in self.manifest.include:
                self._includes.append(include)
            self.__args = manifest.args
        else:
            self.__args = empty_args()

    @property
    def args(self):
        """ Arguments with manifest updated file. """
        if not self.__args_updated:
            for inc in self.include_templates:
                self.__args.update(inc.args)
            self.__args_updated = True
        return self.__args

    def add_ignored(self, args):
        """ Add ignore pattern. """
        if isinstance(args, list):
            for elem in args:
                self._ignored.append(elem)
        elif isinstance(args, str):
            self._ignored.append(args)
        else:
            raise AttributeError("Argument should have type 'list' or 'str'")

    def _is_ignored(self, full_path):
        """ Tell if path file is ignored. """
        for ignor in self._ignored:
            if fnmatch.fnmatch(full_path, "*/" + ignor):
                return True
        return False

    def __str__(self):
        return self._name

    @classmethod
    def _get_elements(cls):
        """ Get one of the file's elements. """
        raise NotImplementedError()

    @property
    def name(self):
        """ Name of template. """
        return self._name

    @property
    def include_templates(self):
        """ List of included template. """
        if self._include_templates is None:
            result = []
            for inc in self._includes:
                result.append(self.manager.get_template(inc))
            self._include_templates = result
        return self._include_templates

    @property
    def elements(self):
        """ List of all template's elements. """
        if self._all_elements is None:
            result = self._get_elements()
            for inc in self.include_templates:
                for element in inc.elements:
                    element.template = self
                    result.append(element)
            return result
        return self._all_elements

    @property
    def short_description(self):
        """ Return short description. """
        if self.manifest:
            return self.manifest.short_description
        return "no description"

    @property
    def description(self):
        """ Return long description. """
        if self.manifest:
            return self.manifest.description
        return "no description"

    @classmethod
    def examine_save(cls, path, force=False):
        """ Check if template can be save. """
        if force:
            return
        if os.path.isdir(path):
            raise TemplateError("File '{}' already exist".format(path))

    def print_elements(self):
        """ Print elements on the screen. """
        for element in self.elements:
            element.print_element()

    def save(self, path, project_name=None, force=False):
        """ Save template in path. """
        if self.args.missing_args:
            raise TemplateError("cannot set {} arguments. First is [{}]"
                                "".format(len(self.args.missing_args),
                                          self.args.missing_args[0]))
        if project_name == "":
            project_name = self.name
        elif project_name is None:
            self.print_elements()
            return
        self.examine_save(path, force=force)
        for element in self.elements:
            element.save(path, project_name=project_name)


class TMPSource(abc.ABC):
    """ Class manage one of the containers. """

    def __init__(self, manager):
        self.manager = manager
        self._templates = None

    @classmethod
    def get_all_templates(cls):
        """ Get list of all templates. """
        raise NotImplementedError()

    @property
    def templates(self):
        """ List of all templates. """
        if self._templates is None:
            self._templates = self.get_all_templates()
        return self._templates

    def get_template(self, name):
        """ Get one of templates by id. """
        for template in self.templates:
            if template.name == name:
                assert isinstance(template, Template)
                return template
        return None



#!/usr/bin/env python3
"""
Create new project in defined place..

Example usage
=============

To invoke program type in your console::

    templateme -t <template_name> -p <your_project_name>

or::

    python3 -m templateme -t <template_name> -p <your_project_name>

for more option call program with flag C{--help}
"""

from __future__ import unicode_literals
from __future__ import print_function
import logging
from argparse import ArgumentParser
from templateme import get_version
from templateme.manager import TMPManager
from templateme.abstract import Template
from templateme.abstract import TemplateError


def __option_args(argv=None):
    """
    Parsing argument for command line program.

    @param argv: Option parameters
    @type argv: list
    @return: parsed arguments
    """
    parser = ArgumentParser(description="Create new project from template",
                            prog='templateme')
    parser.add_argument("-v", "--version", action='version',
                        version='%(prog)s {}'.format(get_version()))
    parser.add_argument("-d", "--log", dest="logLevel",
                        choices=['DEBUG', 'INFO', 'WARNING',
                                 'ERROR', 'CRITICAL'],
                        help="Set the logging level")
    parser.add_argument("-p", "--project-name", metavar="NAME",
                        dest="project_name", default="project",
                        help="Project name")
    parser.add_argument("-l", "--list", action="store_true",
                        dest="list", default=False,
                        help="List available templates")
    parser.add_argument("-s", "--short-list", action="store_true",
                        dest="short_list", default=False,
                        help="Templates list wihtout description")
    parser.add_argument("-q", "--quite", action="store_true",
                        dest="quite", default=False,
                        help="Not ask about arguments")
    parser.add_argument("-t", "--temp", metavar="TEMPLATE",
                        dest="template", default="",
                        help="Template name")
    return parser.parse_args(argv)


def set_template_arguments(template):
    """ Set argument for template. """
    for key, value in template.args.items():
        default_str = ""
        if value is not None and value != "":
            default_str = " (default: {})".format(value)
        input_str = input("Write ['{}'] value{}: ".format(key, default_str))
        if input_str == "":
            input_str = value
        template.set_argument(key, input_str)


def main(argv=None):
    """
    Main function for command line program.

    @param argv: Option parameters
    @type argv: list
    """
    options = __option_args(argv)
    logging.basicConfig(format='%(asctime)s - %(name)s - '
                               '%(levelname)s - %(message)s',
                        level=options.logLevel)
    manager = TMPManager(options.project_name)
    if options.list or options.short_list:
        templates = manager.get_all_templates()
        for temp in templates:
            if options.short_list:
                print("{} ".format(temp))
            else:
                print("* {} - {}".format(temp, temp.short_description))
        exit(0)
    elif options.template == "":
        print("You should define template's name\nSee --help for more information")
        exit(1)

    template = manager.get_template(options.template)
    assert isinstance(template, Template)
    if template is None:
        print("There are not template name: ", options.template)
        exit(1)
    if not options.quite:
        set_template_arguments(template)
    try:
        template.save(options.project_name, options.project_name)
    except TemplateError as ex:
        print("Cannot save: ", ex)
        exit(2)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
Testing module templateme.

TestCase checking module files. It should be conform to PEP8.

@see: L{templateme}
"""

from __future__ import unicode_literals
import unittest
import os
import sys
from io import StringIO
import pep8
from pylint.lint import Run as run_pylint
import templateme


# This is tested class. Can have too many method
class TestTemplateMeModule(unittest.TestCase):  # pylint: disable=R0904
    """ Module testsCase. """
    @staticmethod
    def __get_sources_file():
        """ Get all paths to source files in templateme module. """

        def recursive_checker(main_directory):
            """ Recursive function to find all source file in direcotry. """
            result = []
            for source_file in os.listdir(main_directory):
                if (source_file.endswith(".py") and
                        os.path.isfile(os.path.join(main_directory,
                                                    source_file))):
                    result.append(os.path.join(main_directory, source_file))
                elif source_file == "templates":
                    continue
                elif os.path.isdir(os.path.join(main_directory, source_file)):
                    result = (result +
                              recursive_checker(os.path.join(main_directory,
                                                             source_file)))
            return result
        return recursive_checker(templateme.__path__[0])

    def test_pep8(self):
        """ Tests that we conform to PEP8. """
        pep8_style = pep8.StyleGuide(paths=['--ignore=E501'])
        # Disable E501 code (line too long). It should be enabled after fixed.
        result = pep8_style.check_files(TestTemplateMeModule.__get_sources_file())
        self.assertEqual(result.total_errors, 0,
                         "Found code style errors (and warnings).")

    def test_pylint(self):
        """ Tests that we conform to Pylint's rules. """
        status = 0
        output = None
        try:
            old_stdout = sys.stdout
            old_stderr = sys.stderr
            sys.stdout = output = StringIO()
            sys.stderr = StringIO()
            run_pylint(['-d', 'I0011,R0801,R0902,R0903,R0913,R0921', 'templateme'])
        except SystemExit as ex:
            status = int(ex.code)
        finally:
            sys.stdout = old_stdout
            sys.stderr = old_stderr

        if status != 0 and output is not None:
            print(output.getvalue())
        self.assertEqual(status, 0, "[Pylint] Found code style errors"
                                    " (and warnings).")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
"""
Module of template's manager.
"""
import os
import logging
import datetime
from templateme.containers.resource import ResourceSource
from templateme.containers.path import PathSource
from templateme.configuration import Configuration


class TMPManagerError(Exception):
    """ Manager error. """


class TMPManager:
    """ Template manager. """

    def __init__(self, name="Project", debug=False):
        self.plugins = []
        self.__config = Configuration(debug=debug)

        self.__register_sources()
        self.name = name

    def __register_sources(self):
        """ Register all plugins needed by manager. """
        self.plugins.append(ResourceSource(manager=self, source_dir="templates"))
        if self.__config.debug:
            logging.debug("Debug mode, find templates only from package")
            return
        template_path_tab = ["/etc/templateme", "~/.config/templateme"]
        template_path_tab = template_path_tab + self.__config.localizations
        for template_path in template_path_tab:
            template_path = os.path.expanduser(template_path)
            if os.path.isdir(template_path):
                path_source = PathSource(manager=self, path=template_path)
                self.plugins.append(path_source)

    def get_all_templates(self):
        """ Return all of available templates from all of containers. """
        result = []
        for plug in self.plugins:
            logging.debug("get_all_templates [%s]", plug)
            for temp in plug.get_all_templates():
                result.append(temp)
        return result

    def get_template(self, name):
        """ Return template by name. """
        template = None
        for plug in self.plugins:
            template = plug.get_template(name)
            if template is not None:
                break
        return template

    def render_template_txt(self, txt, template):
        """ Rendering template to text format. """
        result = txt
        changes = {
            'EMAIL': self.__config.get_val('email'),
            'AUTHOR': self.__config.get_val('author'),
            'YEAR': datetime.datetime.now().strftime("%Y"),
            'NAME': self.name if self.name is not None else "console"
        }
        template.args.add_values(changes)
        for key in template.args.all:
            elem = template.args.get_argument(key)
            result = result.replace("%{}%".format(elem.name), elem.value)
        return result



#!/usr/bin/env python3
"""
Module for template from resource.
"""
from os.path import join
import logging
import pkg_resources
import templateme
from templateme.manifest import Manifest
from templateme.manifest import ManifestError
import templateme.containers.abstract


class ResourceElement(templateme.containers.abstract.TMPElement):
    """ Class with template's element file from path. """

    def __init__(self, path, template, localization, name):
        templateme.containers.abstract.TMPElement.__init__(self, path, template)
        self.localization = localization
        self.__load_txt = None
        self.name = name

    def load_txt(self):
        """ Load information from file. """
        if self.__load_txt is None:
            full_path = join(self.localization, self.name, self.path)
            self.__load_txt = pkg_resources.resource_string("templateme", full_path).decode("utf-8")
        return self.__load_txt


class ResourceTemplate(templateme.containers.abstract.Template):
    """ Class with template from path. """

    def __init__(self, name, manager, package_localization="templates"):
        manifest = None
        try:
            manifest_path = join(package_localization, name, "manifest.json")
            json_str = pkg_resources.resource_string("templateme", manifest_path)
            manifest = Manifest.create_from_string(json_str.decode("utf-8"), self)
        except ManifestError:
            pass
        templateme.containers.abstract.Template.__init__(self, name, manager, manifest)
        self.__package_localization = package_localization

    def __search_in_dir(self, package_localization, name, path_in, elements):
        """ Search in directory. """
        where_look = join(package_localization, name)
        where_look = join(where_look, path_in)
        for directory_name in pkg_resources.resource_listdir("templateme", where_look):
            directory_path = join(where_look, directory_name)
            if self._is_ignored(directory_path):
                continue
            if pkg_resources.resource_isdir("templateme", directory_path):
                self.__search_in_dir(package_localization,
                                     name,
                                     join(path_in, directory_name),
                                     elements)
            else:
                resource_element = ResourceElement(join(path_in, directory_name),
                                                   self,
                                                   package_localization,
                                                   name)
                elements.append(resource_element)

    def _get_elements(self):
        elements = []
        self.__search_in_dir(self.__package_localization, self.name, "", elements)
        return elements


class ResourceSource(templateme.containers.abstract.TMPSource):
    """ Class with path source from directory. """

    def __init__(self, manager, source_dir="templates"):
        templateme.containers.abstract.TMPSource.__init__(self, manager)
        self.__templates_elem = source_dir

    def get_all_templates(self):
        """ Return all templates from directory. """
        templates = []
        logging.debug(self.__templates_elem)
        for temp_name in pkg_resources.resource_listdir("templateme", self.__templates_elem):
            templates.append(ResourceTemplate(temp_name, self.manager, self.__templates_elem))
        return templates



#!/usr/bin/env python3
"""
Template Me - module with template for your favourite projects.
"""


def get_version():
    """ Get version of protemp package. """
    return "0.0.7"

__version__ = get_version()



#!/usr/bin/env python3
"""
CommandLine interface for test.

For checking tests call::

    python -m templateme.tests
"""

# pylint: disable=W0611
import unittest
from templateme.tests import TestTemplateMeModule
from templateme.tests.console import TestConsoleModule
from templateme.tests.templates import TestTemplatesModule
from templateme.tests.arguments import TestArgumentsModule


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
"""
Testing module console.
"""

import unittest
from templateme.arguments import Argument


# This is tested class. Can have too many method
class TestArgumentsModule(unittest.TestCase):  # pylint: disable=R0904
    """ Module testsCase. """

    def test_argument(self):
        """ Tests if Argument object work good. """
        first_argument = Argument("first_Argument")
        first_argument.value = "first_Value"
        self.assertEqual(first_argument.value, "first_Value")
        self.assertEqual(first_argument.name, "FIRST_ARGUMENT")
        first_argument.value = "second Value"
        self.assertEqual(first_argument.value, "second Value")

    def test_argument_require(self):
        """ Tests if require mechanism works good. """
        default_arg = Argument("element")
        self.assertEqual(default_arg.is_set, False)
        default_arg.value = "test"
        self.assertEqual(default_arg.is_set, True)

        default_arg = Argument("element", required=False)
        self.assertEqual(default_arg.is_set, True)
        default_arg.value = "test"
        self.assertEqual(default_arg.is_set, True)

        default_arg = Argument("element", default="test")
        self.assertEqual(default_arg.value, "test")
        self.assertEqual(default_arg.is_set, False)
        default_arg.value = "test2"
        self.assertEqual(default_arg.value, "test2")
        self.assertEqual(default_arg.is_set, True)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
"""
Configuration files for TemplateMe!

The module contains class which allows configuration terminal console
and all default option in TemplateMe.
"""

from configparser import ConfigParser
import os


class Configuration:
    """ Configuration class. """

    default_config = """
[global]
;; Default email name
email=user@unknown.com
;; default user name
author=user
;; default license
license=MIT
;; additional localizations separated by ':' sign
templates=
    """

    def __init__(self, debug=False):
        if debug:
            config_files = []
        else:
            config_files = [
                '/etc/templateme/config.ini',
                '~/.config/templateme/config.ini',
                '~/.templateme'
            ]

        default_config = ConfigParser()
        default_config.read_string(Configuration.default_config)
        self.__configurations = [default_config]

        for elem in config_files:
            path_file = os.path.expanduser(elem)
            if not os.path.isfile(path_file):
                continue

            with open(path_file, "r") as config_file:
                new_config = ConfigParser()
                new_config.read_file(config_file)
                self.__configurations.append(new_config)

        self.__debug = debug

    def get_val(self, key, section="global", default=None):
        """ Get value of option. """
        for config in reversed(self.__configurations):
            if section in config:
                if key in config[section]:
                    return config[section][key]
        return default

    @property
    def debug(self):
        """ Check if configuration is from debug. """
        return self.__debug

    @property
    def localizations(self):
        """ Localization of all place where application should look for templates. """
        result = []
        locations = self.get_val("templates", default="")
        for location in locations.split(":"):
            path_location = os.path.expanduser(location)
            if os.path.isdir(path_location):
                result.append(path_location)
        return result



#!/usr/bin/env python3
"""
Setup file.
"""

from __future__ import unicode_literals
import os
import sys
from setuptools import setup
from setuptools import find_packages


def read_description(module_name):
    module_doc = __import__(module_name).__doc__.splitlines()
    result = ""
    for line in module_doc:
        if line:
            result = line
            break
    return result

CURRENT_PYTHON = sys.version_info[:2]
REQUIRED_PYTHON = (3, 5)

if CURRENT_PYTHON < REQUIRED_PYTHON:
    sys.stderr.write("""
==========================
Unsupported Python version
==========================
This version of Templateme requires Python {}.{}, but you're trying to
install it on Python {}.{}.
This may be because you are using a version of pip that doesn't
understand the python_requires classifier. Make sure you
have python 3.5 or newer, then try again:
    $ python3 -m pip install --upgrade pip setuptools
    $ pip3 install templateme
""".format(*(REQUIRED_PYTHON + CURRENT_PYTHON)))
    sys.exit(1)
    
EXCLUDE_FROM_PACKAGES = ['templateme.templates']

version = __import__('templateme').get_version()

def package_data():
    result = []
    for root, dirs, files in os.walk('./templateme/templates'):
        for name in files:
            full_path = os.path.join(root, name)
            result.append(full_path.replace("./templateme/", "./"))
    return { 'templateme' : result }

setup(
    name='templateme',
    version=version,
    license='GNU',
    author="Rafal Kobel",
    author_email="rafalkobel@rafyco.pl",
    python_requires='>={}.{}'.format(*REQUIRED_PYTHON),
    description=read_description('templateme'),
    long_description=open("README.rst").read(),
    url="https://github.com/rafyco/templateme",
    include_package_data=True,
    package_dir={'templateme': 'templateme'},
    package_data=package_data(),
    test_suite='templateme.tests.__main__',
    classifiers=[
        'Environment :: Console',
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'License :: OSI Approved :: MIT License',
        'Topic :: Terminals'
    ],
    install_requires=[
        'pylint',
        'mock',
        'pep8==1.7.1',
        'Sphinx==1.8.4'
    ],
    entry_points={
        'console_scripts': [
            'templateme = templateme.console:main',
        ]
    },
    command_options={
        'build_sphinx': {
            'project': ('setup.py', "templateme"),
            'version': ('setup.py', version),
            'release': ('setup.py', version),
            'source_dir': ('setup.py', "docs")
        }
    },
    packages=find_packages(exclude=EXCLUDE_FROM_PACKAGES),
    platforms="Any",
    keywords="template, programming, fast",
    project_urls={
        "Bug Reports": "https://github.com/rafyco/templateme/issues",
        "Source": "https://github.com/rafyco/templateme",
        "Documentation": "https://templateme.readthedocs.io/en/latest/"
    }
)



#!/usr/bin/env python3
import os
import sys
from setuptools import setup


def read_description(module_name):
    module_doc = __import__(module_name).__doc__.splitlines()
    result = ""
    for line in module_doc:
        if line:
            result = line
            break
    return result

CURRENT_PYTHON = sys.version_info[:2]
REQUIRED_PYTHON = (3, 4)

if CURRENT_PYTHON < REQUIRED_PYTHON:
    sys.stderr.write("""
==========================
Unsupported Python version
==========================
This version of %MODULE% requires Python {}.{}, but you're trying to
install it on Python {}.{}.
This may be because you are using a version of pip that doesn't
understand the python_requires classifier. Make sure you
have python 3.4 or newer, then try again:
    $ python3 -m pip install --upgrade pip setuptools
    $ pip3 install %MODULE% 
""".format(*(REQUIRED_PYTHON + CURRENT_PYTHON)))
    sys.exit(1)

EXCLUDE_FROM_PACKAGES = ['%MODULE%.%MODULE%']

version = __import__('%MODULE%').get_version()


setup(
    name='%MODULE%',
    version=version,
    license='GNU',
    author="%AUTHOR%",
    author_email="%EMAIL%",
    python_requires='>={}.{}'.format(*REQUIRED_PYTHON),
    description=read_description('%MODULE%'),
    long_description=open("README.rst").read(),
    url="%URL%",
    include_package_data=True,
    package_dir={'%MODULE%': '%MODULE%'},
    package_data=package_data(),
    test_suite='%MODULE%.tests.__main__',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    install_requires=[
        'pylint',
        'pep8'
    ],
    keywords=""
)



#!/usr/bin/env python3
"""
Module for template from path.
"""
import os
import logging
import templateme
from templateme.manifest import Manifest
from templateme.manifest import ManifestError
import templateme.containers.abstract


class TemplateError(Exception):
    """ Problem with template. """


class PathElement(templateme.containers.abstract.TMPElement):
    """ Class with template's element file from path. """

    def __init__(self, path, localization, template):
        templateme.containers.abstract.TMPElement.__init__(self, path, template)
        self.localization = localization
        self._load_txt = ""

    def load_txt(self):
        """ Load information from file. """
        if self._load_txt == "":
            with open(self.localization) as file_ob:
                self._load_txt = file_ob.read()
        return self._load_txt


class PathTemplate(templateme.containers.abstract.Template):
    """ Class with template from path. """

    def __init__(self, path, name, manager):
        manifest = None
        try:
            manifest = Manifest.create_from_file(os.path.join(path, name, "manifest.json"), self)
        except ManifestError:
            pass
        except PermissionError as ex:
            logging.warning("Cannot read manifest file from %s,\n"
                            "Please try change permissions", path)
            raise TemplateError("Permission error", ex)
        templateme.containers.abstract.Template.__init__(self, name, manager, manifest)
        self._path = os.path.join(path, name)

    def _get_elements(self):
        elements = []
        for root, _, files in os.walk(self._path):
            for name in files:
                full_path = os.path.join(root, name)
                if self._is_ignored(full_path):
                    continue
                path_element = os.path.relpath(full_path, self._path)
                element = PathElement(path_element, full_path, self)
                elements.append(element)
        return elements


class PathSource(templateme.containers.abstract.TMPSource):
    """ Class with path source from directory. """

    def __init__(self, manager, path):
        templateme.containers.abstract.TMPSource.__init__(self, manager)
        self._path = path

    def get_all_templates(self):
        """ Return all templates from directory. """
        logging.debug("Path source, get_templates [%s]", self._path)
        templates = []
        logging.debug(self._path)
        templates_path = os.listdir(self._path)
        for temp_file in templates_path:
            if not os.path.isdir(os.path.join(self._path, temp_file)):
                continue
            try:
                templates.append(PathTemplate(self._path, temp_file, self.manager))
            except TemplateError:
                logging.warning("Problem with template crated [%s]"
                                "", os.path.join(self._path, temp_file))
        return templates
